@@ -6,7 +6,7 @@ IRI - Internationalized Resource Identifiers
 
 =head1 VERSION
 
-This document describes IRI version 0.009_01
+This document describes IRI version 0.010_01
 
 =head1 SYNOPSIS
 
@@ -80,7 +80,7 @@ Returns the respective component of the parsed IRI.
 	package IRI; 
 	use v5.10.1;
 	use warnings;
-	our $VERSION	= '0.009_01';
+	our $VERSION	= '0.010_01';
 	use Moo;
 	use MooX::HandlesVia;
 	use Types::Standard qw(Str InstanceOf HashRef Bool);
@@ -112,8 +112,10 @@ Returns the respective component of the parsed IRI.
 		isa		=> HashRef,
 		lazy	=> 1,
 		builder	=> '_resolved_components',
+		predicate => 1,
 		handles_via	=> 'Hash',
 		handles	=> {
+			authority	=>  [ accessor => 'authority' ],
 			scheme		=>  [ accessor => 'scheme' ],
 			host		=>  [ accessor => 'host' ],
 			port		=>  [ accessor => 'port' ],
@@ -135,10 +137,15 @@ Returns the respective component of the parsed IRI.
 	
 	sub BUILD {
 		my $self	= shift;
-		unless ($self->lazy) {
-			my $comp	= $self->_parse_components($self->value);
+		if ($self->has_resolved_components) {
+		  $self->_set_components($self->resolved_components);
+		  $self->_initialized(1);
+		} else {
+		  unless ($self->lazy) {
+			 my $comp	= $self->_parse_components($self->value);
+		  }
 		}
-	}
+	 }
 	
 	before [qw(components as_string abs resolved_components scheme host port user path fragment query)] => sub {
 		my $self	= shift;
@@ -224,7 +231,7 @@ Returns the respective component of the parsed IRI.
 	my $iregname		= qr<(${iunreserved}|${pctencoded}|${subdelims})*>o;
 	my $ihost			= qr<(?<host>${IPliteral}|${IPv4address}|${iregname})>o;
 	my $iuserinfo		= qr<(?<user>(${iunreserved}|${pctencoded}|${subdelims}|:)*)>o;
-	my $iauthority		= qr<(${iuserinfo}@)?${ihost}(:${port})?>o;
+	my $iauthority		= qr<(?<authority>(${iuserinfo}@)?${ihost}(:${port})?)>o;
 	my $irelativepart	= qr<
 							(//${iauthority}${ipathabempty})
 						|	${ipathabsolute}
@@ -394,9 +401,112 @@ Returns the respective component of the parsed IRI.
 		return $value;
 	}
 
+=item C<< rel ( $base ) >>
+
+Returns a new relative IRI object which, when resolved against the C<< $base >>
+IRI, is equal to this IRI.
+
+=cut
+
+	sub rel {
+		# based on code in URI <https://metacpan.org/source/OALDERS/URI-1.76/lib/URI/_generic.pm#L191>
+		my $self	= shift;
+		my $base	= shift;
+		my $rel		= IRI->new(value => $self->abs);
+		
+		if (($base->scheme // '') ne ($rel->scheme // '')) {
+			return IRI->new(value => $rel->abs);
+		}
+
+		my $scheme = $rel->scheme;
+		my $auth   = $rel->authority;
+		my $path   = $rel->path;
+		
+		if (!defined($scheme) and !defined($auth)) {
+			return $rel;
+		}
+
+		my $bscheme = $base->scheme;
+		my $bauth   = $base->authority;
+		my $bpath   = $base->path;
+
+		for ($bscheme, $bauth, $auth) {
+			$_ = '' unless defined($_);
+		}
+		
+		if ($scheme eq $bscheme) {
+			$rel->scheme(undef);
+		}
+		
+		unless ($scheme eq $bscheme and $auth eq $bauth) {
+			return IRI->new(value => $rel->_abs);
+		}
+
+		for ($path, $bpath) {
+			$_ = "/$_" unless m{^/};
+		}
+
+		# Make it relative by eliminating:
+		# the scheme,
+		$rel->scheme(undef);
+
+		# ... and authority
+		$rel->host(undef);
+		$rel->port(undef);
+		$rel->user(undef);
+		
+		
+		my @rparts	= split('/', $path);
+		my @bparts	= split('/', $bpath);
+		shift(@rparts);
+		shift(@bparts);
+		if (scalar(@rparts) and (scalar(@bparts) and $rparts[0] ne $bparts[0])) {
+			# use an absolute path, because $rel differs from $base at the very beginning
+		} else {
+			# This loop is based on code from Nicolai Langfeldt <janl@ifi.uio.no>.
+			# First we calculate common initial path components length ($li).
+			my $li = 1;
+			while (1) {
+				my $i = index($path, '/', $li);
+				last if $i < 0 ||
+					$i != index($bpath, '/', $li) ||
+					substr($path,$li,$i-$li) ne substr($bpath,$li,$i-$li);
+				$li=$i+1;
+			}
+		
+			# then we nuke it from both paths
+			substr($path, 0,$li) = '';
+			substr($bpath,0,$li) = '';
+
+
+			if ($path eq $bpath) {
+				$rel->path('');
+				if (defined($rel->query) and defined($base->query)) {
+					if ($rel->query eq $base->query) {
+						$rel->query(undef);
+					} else {
+						#
+					}
+				} elsif (defined($rel->query)) {
+					#
+				} elsif (defined($base->query)) {
+					$rel->path($path);
+				} else {
+					#
+				}
+			} else {
+				# Add one "../" for each path component left in the base path
+				$path = ('../' x $bpath =~ tr|/|/|) . $path;
+				$path = "./" if $path eq '';
+				$rel->path($path);
+			}
+		}
+		return IRI->new(value => $rel->_abs);
+	}
+
 	sub as_string {
 		my $self	= shift;
-		if ($self->has_base) {
+		if ($self->has_base || $self->has_resolved_components) {
 			return $self->abs;
 		} else {
 			return $self->value;
